@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2003, 2004 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -24,7 +24,7 @@
 extern "C" {
 #endif
 
-#define MU_ERR_BASE            0x1000
+#define MU_ERR_BASE                 0x1000
 
 #define MU_ERR_FAILURE              (MU_ERR_BASE + 1)
 #define MU_ERR_NO_HANDLER           (MU_ERR_BASE + 2)
@@ -66,6 +66,37 @@ extern "C" {
 #define MU_ERR_TCP_NO_PORT          (MU_ERR_BASE + 26)
 
 #define MU_ERR_BAD_2047_INPUT       (MU_ERR_BASE + 27)
+
+#define MU_ERR_NOUSERNAME           (MU_ERR_BASE + 28)
+#define MU_ERR_NOPASSWORD           (MU_ERR_BASE + 29)
+
+#define MU_ERR_UNSAFE_PERMS         (MU_ERR_BASE + 30)
+  /* unsafe file permissions */
+
+#define MU_ERR_BAD_AUTH_SCHEME      (MU_ERR_BASE + 31)
+  /* Unsupported authentication scheme */
+
+#define MU_ERR_AUTH_FAILURE         (MU_ERR_BASE + 32)
+  /* Authentication failed */
+
+#define MU_ERR_PROCESS_NOEXEC       (MU_ERR_BASE + 33)
+  /* Unable to execute subprocess */
+#define MU_ERR_PROCESS_EXITED       (MU_ERR_BASE + 34)
+  /* Subprocess exited with a non-zero status */
+#define MU_ERR_PROCESS_SIGNALED     (MU_ERR_BASE + 35)
+  /* Subprocess exited on signal */
+
+#define MU_ERR_PROCESS_UNKNOWN_FAILURE (MU_ERR_BASE + 36)
+  /* Unknown failure while executing subprocess */
+
+#define MU_ERR_CONN_CLOSED	    (MU_ERR_BASE + 37)
+  /* Connection closed by remote host */
+#define MU_ERR_PARSE		    (MU_ERR_BASE + 38)
+  /* Parse error */
+#define MU_ERR_NOENT                (MU_ERR_BASE + 39)  
+  /* Requested item not found */
+#define MU_ERR_BUFSPACE             (MU_ERR_BASE + 40)
+  /* Not enough buffer space */
   
 const char *mu_errname __P((int e));
 const char *mu_strerror __P((int e));
