@@ -1,5 +1,5 @@
 
-/*  A Bison parser, made from /home/gray/mailutils/libsieve/./sieve.y
+/*  A Bison parser, made from /home/gray/gnu/mailutils/libsieve/./sieve.y
     by GNU Bison version 1.28  */
 
 #define YYBISON 1  /* Identify Bison output.  */
@@ -48,9 +48,10 @@ sieve_machine_t sieve_machine;
 int sieve_error_count;
 
 static void branch_fixup __P((size_t start, size_t end));
- 
+int sieve_yyerror (char *s);
+int sieve_yylex (); 
 
-#line 34 "sieve.y"
+#line 35 "sieve.y"
 typedef union {
   char *string;
   size_t number;
@@ -150,11 +151,11 @@ static const short sieve_yyrhs[] = {    -1,
 
 #if YYDEBUG != 0
 static const short sieve_yyrline[] = { 0,
-    70,    71,    75,    76,    79,    87,    89,    97,   103,   114,
-   131,   135,   138,   144,   154,   162,   170,   176,   183,   194,
-   202,   204,   208,   212,   219,   225,   244,   251,   270,   274,
-   277,   282,   289,   293,   297,   301,   305,   311,   316,   319,
-   325,   330
+    71,    72,    76,    77,    80,    88,    90,    98,   104,   115,
+   132,   136,   139,   145,   155,   163,   171,   177,   184,   195,
+   203,   205,   209,   213,   220,   226,   245,   252,   271,   275,
+   278,   283,   290,   294,   298,   302,   306,   312,   317,   320,
+   326,   331
 };
 #endif
 
@@ -783,11 +784,11 @@ sieve_yyreduce:
   switch (sieve_yyn) {
 
 case 2:
-#line 72 "sieve.y"
+#line 73 "sieve.y"
 { /* to placate bison */ ;
     break;}
 case 5:
-#line 80 "sieve.y"
+#line 81 "sieve.y"
 {
 		 sieve_require (sieve_yyvsp[-1].list);
 		 /*  All the items in $2 are registered in memory_pool,
@@ -797,7 +798,7 @@ case 5:
 	       ;
     break;}
 case 7:
-#line 90 "sieve.y"
+#line 91 "sieve.y"
 {
 		 sieve_machine->prog[sieve_yyvsp[-2].pc].pc = sieve_yyvsp[0].branch.begin - sieve_yyvsp[-2].pc - 1;
 		 if (sieve_yyvsp[0].branch.branch)
@@ -805,13 +806,13 @@ case 7:
 	       ;
     break;}
 case 8:
-#line 98 "sieve.y"
+#line 99 "sieve.y"
 {
 		 sieve_yyval.pc = sieve_machine->pc;
 	       ;
     break;}
 case 9:
-#line 104 "sieve.y"
+#line 105 "sieve.y"
 {
 		 if (sieve_yyvsp[0].branch.begin)
 		   sieve_machine->prog[sieve_yyvsp[0].branch.cond].pc =
@@ -824,7 +825,7 @@ case 9:
 	       ;
     break;}
 case 10:
-#line 115 "sieve.y"
+#line 116 "sieve.y"
 {
 		 if (sieve_yyvsp[-2].branch.begin)
 		   {
@@ -841,13 +842,13 @@ case 10:
 	       ;
     break;}
 case 11:
-#line 132 "sieve.y"
+#line 133 "sieve.y"
 {
 		 sieve_yyval.branch.begin = 0;
 	       ;
     break;}
 case 13:
-#line 139 "sieve.y"
+#line 140 "sieve.y"
 {
 		 sieve_yyval.branch.begin = sieve_yyvsp[-2].pc; 
 		 sieve_yyval.branch.branch = sieve_yyvsp[-3].pc;
@@ -855,7 +856,7 @@ case 13:
 	       ;
     break;}
 case 14:
-#line 145 "sieve.y"
+#line 146 "sieve.y"
 {
 		 sieve_machine->prog[sieve_yyvsp[-4].branch.cond].pc = sieve_yyvsp[-2].pc - sieve_yyvsp[-4].branch.cond - 1;
 		 sieve_machine->prog[sieve_yyvsp[-3].pc].pc = sieve_yyvsp[-4].branch.branch;
@@ -865,7 +866,7 @@ case 14:
 	       ;
     break;}
 case 15:
-#line 155 "sieve.y"
+#line 156 "sieve.y"
 {
 		 sieve_code_instr (instr_branch);
 		 sieve_yyval.pc = sieve_machine->pc;
@@ -873,7 +874,7 @@ case 15:
 	       ;
     break;}
 case 16:
-#line 163 "sieve.y"
+#line 164 "sieve.y"
 {
 		 sieve_code_instr (instr_branch);
 		 sieve_yyval.pc = sieve_machine->pc;
@@ -881,13 +882,13 @@ case 16:
 	       ;
     break;}
 case 17:
-#line 171 "sieve.y"
+#line 172 "sieve.y"
 {
 		 sieve_yyval.pc = sieve_yyvsp[-1].pc;
 	       ;
     break;}
 case 18:
-#line 177 "sieve.y"
+#line 178 "sieve.y"
 {
 		 sieve_yyval.pclist.start = sieve_yyval.pclist.end = sieve_machine->pc;
 		 if (sieve_code_instr (instr_brz)
@@ -896,7 +897,7 @@ case 18:
 	       ;
     break;}
 case 19:
-#line 184 "sieve.y"
+#line 185 "sieve.y"
 {
 		 sieve_machine->prog[sieve_yyvsp[-2].pclist.end+1].pc = sieve_machine->pc;
 		 sieve_yyvsp[-2].pclist.end = sieve_machine->pc;
@@ -907,7 +908,7 @@ case 19:
 	       ;
     break;}
 case 20:
-#line 195 "sieve.y"
+#line 196 "sieve.y"
 {
 		 sieve_code_instr (instr_brz);
 		 sieve_yyval.pc = sieve_machine->pc;
@@ -915,36 +916,36 @@ case 20:
 	       ;
     break;}
 case 21:
-#line 203 "sieve.y"
+#line 204 "sieve.y"
 { /* to placate bison */ ;
     break;}
 case 22:
-#line 205 "sieve.y"
+#line 206 "sieve.y"
 {
 		 sieve_code_anyof (sieve_yyvsp[-1].pclist.start);
 	       ;
     break;}
 case 23:
-#line 209 "sieve.y"
+#line 210 "sieve.y"
 {
 		 sieve_code_allof (sieve_yyvsp[-1].pclist.start);
 	       ;
     break;}
 case 24:
-#line 213 "sieve.y"
+#line 214 "sieve.y"
 {
 		 if (sieve_code_instr (instr_not))
 		   YYERROR;
 	       ;
     break;}
 case 25:
-#line 220 "sieve.y"
+#line 221 "sieve.y"
 {
 		 sieve_yyval.pc = sieve_machine->pc;
 	       ;
     break;}
 case 26:
-#line 226 "sieve.y"
+#line 227 "sieve.y"
 {
 		 sieve_register_t *reg = sieve_test_lookup (sieve_machine,
 							    sieve_yyvsp[0].command.ident);
@@ -963,14 +964,14 @@ case 26:
 	       ;
     break;}
 case 27:
-#line 245 "sieve.y"
+#line 246 "sieve.y"
 {
 		 sieve_yyval.command.ident = sieve_yyvsp[-1].string;
 		 sieve_yyval.command.args = sieve_yyvsp[0].list;
 	       ;
     break;}
 case 28:
-#line 252 "sieve.y"
+#line 253 "sieve.y"
 {
 		 sieve_register_t *reg = sieve_action_lookup (sieve_machine,
 							      sieve_yyvsp[0].command.ident);
@@ -989,77 +990,77 @@ case 28:
 	       ;
     break;}
 case 29:
-#line 271 "sieve.y"
+#line 272 "sieve.y"
 {
 		 sieve_yyval.list = NULL;
 	       ;
     break;}
 case 31:
-#line 278 "sieve.y"
+#line 279 "sieve.y"
 {
 		 list_create (&sieve_yyval.list);
 		 list_append (sieve_yyval.list, sieve_yyvsp[0].value);
 	       ;
     break;}
 case 32:
-#line 283 "sieve.y"
+#line 284 "sieve.y"
 {
 		 list_append (sieve_yyvsp[-1].list, sieve_yyvsp[0].value);
 		 sieve_yyval.list = sieve_yyvsp[-1].list;
 	       ;
     break;}
 case 33:
-#line 290 "sieve.y"
+#line 291 "sieve.y"
 {
 		 sieve_yyval.value = sieve_value_create (SVT_STRING_LIST, sieve_yyvsp[0].list);
 	       ;
     break;}
 case 34:
-#line 294 "sieve.y"
+#line 295 "sieve.y"
 {
 		 sieve_yyval.value = sieve_value_create (SVT_STRING, sieve_yyvsp[0].string);
                ;
     break;}
 case 35:
-#line 298 "sieve.y"
+#line 299 "sieve.y"
 {
 		 sieve_yyval.value = sieve_value_create (SVT_STRING, sieve_yyvsp[0].string);
 	       ;
     break;}
 case 36:
-#line 302 "sieve.y"
+#line 303 "sieve.y"
 {
 		 sieve_yyval.value = sieve_value_create (SVT_NUMBER, &sieve_yyvsp[0].number);
 	       ;
     break;}
 case 37:
-#line 306 "sieve.y"
+#line 307 "sieve.y"
 {
 		 sieve_yyval.value = sieve_value_create (SVT_TAG, sieve_yyvsp[0].string);
 	       ;
     break;}
 case 38:
-#line 312 "sieve.y"
+#line 313 "sieve.y"
 {
 		 list_create (&sieve_yyval.list);
 		 list_append (sieve_yyval.list, sieve_yyvsp[0].string);
 	       ;
     break;}
 case 40:
-#line 320 "sieve.y"
+#line 321 "sieve.y"
 {
 		 sieve_yyval.list = sieve_yyvsp[-1].list;
 	       ;
     break;}
 case 41:
-#line 326 "sieve.y"
+#line 327 "sieve.y"
 {
 		 list_create (&sieve_yyval.list);
 		 list_append (sieve_yyval.list, sieve_yyvsp[0].string);
 	       ;
     break;}
 case 42:
-#line 331 "sieve.y"
+#line 332 "sieve.y"
 {
 		 list_append (sieve_yyvsp[-2].list, sieve_yyvsp[0].string);
 		 sieve_yyval.list = sieve_yyvsp[-2].list;
@@ -1287,7 +1288,7 @@ sieve_yyerrhandle:
     }
   return 1;
 }
-#line 337 "sieve.y"
+#line 338 "sieve.y"
 
 
 int
@@ -1450,20 +1451,28 @@ sieve_machine_destroy (sieve_machine_t *pmach)
   list_destroy (&mach->action_list);
   list_destroy (&mach->test_list);
   list_destroy (&mach->comp_list);
+  list_destroy (&mach->source_list);
   sieve_slist_destroy (&mach->memory_pool);
   free (mach);
   *pmach = NULL;
 }
 
-/* FIXME: When posix thread support is added, sieve_machine_begin() should
-   acquire the global mutex, locking the current compilation session, and
-   sieve_machine_finish() should release it */
+static int
+string_comp (const void *item, const void *value)
+{
+  return strcmp (item, value);
+}
+
 void
-sieve_machine_begin (sieve_machine_t mach)
+sieve_machine_begin (sieve_machine_t mach, const char *file)
 {
   sieve_machine = mach;
   sieve_error_count = 0;
   sieve_code_instr (NULL);
+
+  list_create (&mach->source_list);
+  list_set_comparator (mach->source_list, string_comp);
+  
   sieve_register_standard_actions (mach);
   sieve_register_standard_tests (mach);
   sieve_register_standard_comparators (mach);
@@ -1480,11 +1489,10 @@ sieve_compile (sieve_machine_t mach, const char *name)
 {
   int rc;
   
-  sieve_machine_begin (mach);
+  sieve_machine_begin (mach, name);
 
   if (sieve_lex_begin (name) == 0)
     {
-      sieve_machine->filename = sieve_mstrdup (sieve_machine, name);
       rc = sieve_yyparse ();
       if (sieve_error_count)
 	rc = 1;
