@@ -17,19 +17,24 @@
 
 #include <mail.local.h>
 
-int multiple_delivery;
-int ex_quota_tempfail;
-int exit_code = EX_OK;
-uid_t uid;
-char *quotadbname = NULL;
-int lock_timeout = 300;
+int multiple_delivery;     /* Don't return errors when delivering to multiple
+			      recipients */
+int ex_quota_tempfail;     /* Return temporary failure if mailbox quota is
+			      exceeded. If this variable is not set, mail.local
+			      will return "service unavailable" */
+int exit_code = EX_OK;     /* Exit code to be used */
+uid_t uid;                 /* Current user name */
+char *quotadbname = NULL;  /* Name of mailbox quota database */
 
 /* Debuggig options */
-int debug_level;
-int debug_flags;
-int sieve_debug_flags;
-int sieve_enable_log;
-mu_debug_t mudebug;
+int debug_level;           /* General debugging level */ 
+int debug_flags;           /* Mailutils debugging flags */
+int sieve_debug_flags;     /* Sieve debugging flags */
+int sieve_enable_log;      /* Enables logging of executed Sieve actions */
+char *message_id_header;   /* Use the value of this header as message
+			      identifier when logging Sieve actions */
+mu_debug_t mudebug;        /* Mailutils debugging object */
+
 
 #define MAXFD 64
 #define EX_QUOTA() (ex_quota_tempfail ? EX_TEMPFAIL : EX_UNAVAILABLE)
@@ -43,20 +48,23 @@ void notify_biff __P((mailbox_t mbox, char *name, size_t size));
 
 const char *program_version = "mail.local (" PACKAGE_STRING ")";
 static char doc[] =
-N_("GNU mail.local -- the local MDA"
-   "\v"
-   "Debug flags are:\n"
-   "  g - guimb stack traces\n"
-   "  T - mailutil traces (MU_DEBUG_TRACE)\n"
-   "  P - network protocols (MU_DEBUG_PROT)\n"
-   "  t - sieve trace (MU_SIEVE_DEBUG_TRACE)\n"
-   "  l - sieve action logs\n"
-   "  0-9 - Set mail.local debugging level\n");
+/* TRANSLATORS: Please, preserve the vertical tabulation (^K character)
+   in this message */
+N_("GNU mail.local -- the local MDA\
+\v\
+Debug flags are:\n\
+  g - guimb stack traces\n\
+  T - mailutil traces (MU_DEBUG_TRACE)\n\
+  P - network protocols (MU_DEBUG_PROT)\n\
+  t - sieve trace (MU_SIEVE_DEBUG_TRACE)\n\
+  l - sieve action logs\n\
+  0-9 - Set mail.local debugging level\n");
 
 static char args_doc[] = N_("recipient [recipient ...]");
 
 #define ARG_MULTIPLE_DELIVERY 1
 #define ARG_QUOTA_TEMPFAIL 2
+#define ARG_MESSAGE_ID_HEADER 3
 
 static struct argp_option options[] = 
 {
@@ -73,14 +81,14 @@ static struct argp_option options[] =
 #endif
   { "sieve", 'S', N_("PATTERN"), 0,
     N_("Set name pattern for user-defined sieve mail filters"), 0 },
+  { "message-id-header", ARG_MESSAGE_ID_HEADER, N_("STRING"), 0,
+    N_("Identify messages by the value of this header when logging Sieve actions"), 0 },
 #ifdef WITH_GUILE
   { "source", 's', N_("PATTERN"), 0,
     N_("Set name pattern for user-defined mail filters"), 0 },
 #endif
   { "debug", 'x', N_("FLAGS"), 0,
     N_("Enable debugging"), 0 },
-  { "timeout", 't', N_("NUMBER"), 0,
-    N_("Set timeout for acquiring the lockfile") },
   { NULL,      0, NULL, 0, NULL, 0 }
 };
 
@@ -125,6 +133,10 @@ parse_opt (int key, char *arg, struct argp_state *state)
       ex_quota_tempfail = 1;
       break;
 
+    case ARG_MESSAGE_ID_HEADER:
+      message_id_header = arg;
+      break;
+      
     case 'r':
     case 'f':
       if (from != NULL)
@@ -151,10 +163,6 @@ parse_opt (int key, char *arg, struct argp_state *state)
       sieve_pattern = optarg;
       break;
       
-    case 't':
-      lock_timeout = strtoul (optarg, NULL, 0);
-      break;
-	
     case 'x':
       do
 	{
@@ -222,15 +230,36 @@ _sieve_debug_printer (void *unused, const char *fmt, va_list ap)
 
 static void
 _sieve_action_log (void *user_name,
-		   const char *script, size_t msgno, message_t msg,
+		   const sieve_locus_t *locus, size_t msgno, message_t msg,
 		   const char *action, const char *fmt, va_list ap)
 {
-  size_t uid = 0;
   char *text = NULL;
   
-  message_get_uid (msg, &uid);
-
-  asprintf (&text, _("%s on msg uid %d"), action, uid);
+  if (message_id_header)
+    {
+      header_t hdr = NULL;
+      char *val = NULL;
+      message_get_header (msg, &hdr);
+      if (header_aget_value (hdr, message_id_header, &val) == 0
+	  || header_aget_value (hdr, MU_HEADER_MESSAGE_ID, &val) == 0)
+	{
+	  asprintf (&text, _("%s:%lu: %s on msg %s"),
+		    locus->source_file,
+		    (unsigned long) locus->source_line,
+		    action, val);
+	  free (val);
+	}
+    }
+  if (text == NULL)
+    {
+      size_t uid = 0;
+      message_get_uid (msg, &uid);
+      asprintf (&text, _("%s:%lu: %s on msg uid %d"),
+		locus->source_file,
+		(unsigned long)	locus->source_line,
+		action, uid);
+    }
+  
   if (fmt && strlen (fmt))
     {
       char *diag = NULL;
@@ -275,11 +304,20 @@ main (int argc, char *argv[])
 
   /* Native Language Support */
   mu_init_nls ();
-  
+
+  /* Default locker settings */
+  locker_set_default_flags (MU_LOCKER_PID|MU_LOCKER_RETRY,
+			    mu_locker_set_flags);
+  locker_set_default_retry_timeout (1);
+  locker_set_default_retry_count (300);
+
+  /* Default error code for command line errors */
   mu_argp_error_code = EX_CONFIG;
+  /* Register needed modules */
   MU_AUTH_REGISTER_ALL_MODULES();
   mu_argp_init (program_version, NULL);
   sieve_argp_init ();
+  /* Parse command line */
   mu_argp_parse (&argp, &argc, &argv, 0, argp_capa, &arg_index, NULL);
   
   openlog ("mail.local", LOG_PID, log_facility);
@@ -657,8 +695,6 @@ deliver (mailbox_t imbx, char *name)
     }
   
   mailbox_get_locker (mbox, &lock);
-  locker_set_flags (lock, MU_LOCKER_PID|MU_LOCKER_RETRY);
-  locker_set_retries (lock, lock_timeout);
 
   status = locker_lock (lock);
 
@@ -761,8 +797,15 @@ deliver (mailbox_t imbx, char *name)
 
       if (status)
 	{
-	  mailer_err (_("error writing to mailbox: %s"),
-		      mu_strerror (status));
+	  /* Undo the delivery by truncating the mailbox back to its
+	     original size */
+	  int rc = stream_truncate (ostream, size);
+	  if (rc)
+	    mailer_err (_("error writing to mailbox: %s. Mailbox NOT truncated: %s"),
+			mu_strerror (status), mu_strerror (rc));
+	  else  
+	    mailer_err (_("error writing to mailbox: %s"),
+	  	        mu_strerror (status));
 	}
     }
 
@@ -795,7 +838,7 @@ notify_biff (mailbox_t mbox, char *name, size_t size)
       inaddr.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
       inaddr.sin_port = sp->s_port;
       
-      fd = socket (AF_INET, SOCK_DGRAM, 0);
+      fd = socket (PF_INET, SOCK_DGRAM, 0);
       if (fd < 0)
 	fd = -2; /* Mark failed initialization */
     }
