@@ -33,9 +33,6 @@ mail_from0 (msgset_t *mspec, message_t msg, void *data)
   int cols = util_getcols () - 6;
   int cflag;
   size_t m_size = 0, m_lines = 0;
-  const char *p;
-  struct tm tm;
-  mu_timezone tz;
 
   message_get_header (msg, &hdr);
   if (header_aget_value_unfold (hdr, MU_HEADER_FROM, &from) == 0)
@@ -78,12 +75,32 @@ mail_from0 (msgset_t *mspec, message_t msg, void *data)
     cflag = 'N';
   else
     cflag = ' ';
-  
-  message_get_envelope (msg, &env);
-  envelope_date (env, date, sizeof (date), NULL);
-  p = date;
-  if (mu_parse_ctime_date_time (&p, &tm, &tz) == 0)
-    strftime (date, sizeof(date), "%a %b %e %H:%M", &tm);
+
+  date[0] = 0;
+  if (util_getenv (NULL, "datefield", Mail_env_boolean, 0) == 0
+      && header_get_value (hdr, MU_HEADER_DATE, date, sizeof (date), NULL) == 0)
+    {
+      time_t t;
+      if (mu_parse_date (date, &t, NULL) == 0)
+	{
+	  strftime (date, sizeof(date), "%a %b %e %H:%M", localtime (&t));
+	}
+      else
+	date[0] = 0;
+    }
+
+  if (date[0] == 0)
+    {
+      const char *p;
+      struct tm tm;
+      mu_timezone tz;
+
+      message_get_envelope (msg, &env);
+      envelope_date (env, date, sizeof (date), NULL);
+      p = date;
+      if (mu_parse_ctime_date_time (&p, &tm, &tz) == 0)
+	strftime (date, sizeof(date), "%a %b %e %H:%M", &tm);
+    }
   
   message_size (msg, &m_size);
   message_lines (msg, &m_lines);
