@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003,
+   2004 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -59,7 +60,9 @@ util_ll_add (node *c, int data)
 
 /*
  * free a linked list
+ * Unused so far
  */
+#if 0
 static void
 util_ll_free (node *c)
 {
@@ -71,6 +74,7 @@ util_ll_free (node *c)
       free (c);
     }
 }
+#endif
 
 /*
  * expands command into its command and arguments, then runs command
@@ -552,6 +556,11 @@ util_setenv (const char *variable, void *value, mail_env_data_t type,
 		  fprintf (stderr, "\n");
 		}
 	    }
+	  else if (strcmp (variable, "decode-fallback") == 0)
+	    {
+	      if (mu_set_default_fallback (value))
+		mu_error (_("Incorrect value for decode-fallback"));
+	    }
 	  break;
 	  
 	case Mail_env_boolean:
@@ -644,19 +653,18 @@ util_folder_path (const char *name)
 }
 
 char *
-util_get_sender(int msgno, int strip)
+util_get_sender (int msgno, int strip)
 {
-  header_t header = NULL;
-  address_t addr = NULL;
   message_t msg = NULL;
-  char buffer[512], *p;
+  address_t addr = NULL;
+  char *buf = NULL, *p;
 
   mailbox_get_message (mbox, msgno, &msg);
-  message_get_header (msg, &header);
-  if (header_get_value (header, MU_HEADER_FROM, buffer, sizeof(buffer), NULL)
-      || address_create (&addr, buffer))
+  addr = get_sender_address (msg);
+  if (!addr)
     {
       envelope_t env = NULL;
+      char buffer[512];
       message_get_envelope (msg, &env);
       if (envelope_sender (env, buffer, sizeof (buffer), NULL)
 	  || address_create (&addr, buffer))
@@ -666,7 +674,7 @@ util_get_sender(int msgno, int strip)
 	}
     }
 
-  if (address_get_email (addr, 1, buffer, sizeof(buffer), NULL))
+  if (address_aget_email (addr, 1, &buf))
     {
       util_error (_("can't determine sender name (msg %d)"), msgno);
       address_destroy (&addr);
@@ -675,18 +683,17 @@ util_get_sender(int msgno, int strip)
 
   if (strip)
     {
-      p = strchr (buffer, '@');
+      p = strchr (buf, '@');
       if (p)
 	*p = 0;
     }
 
-  p = strdup (buffer);
   address_destroy (&addr);
-  return p;
+  return buf;
 }
 
 void
-util_slist_print(list_t list, int nl)
+util_slist_print (list_t list, int nl)
 {
   iterator_t itr;
   char *name;
@@ -704,7 +711,7 @@ util_slist_print(list_t list, int nl)
 }
 
 int
-util_slist_lookup(list_t list, char *str)
+util_slist_lookup (list_t list, char *str)
 {
   iterator_t itr;
   char *name;
@@ -1270,7 +1277,7 @@ util_get_message (mailbox_t mbox, size_t msgno, message_t *msg)
   if (msgno > total)
     {
       util_error_range (msgno);
-      return ENOENT;
+      return MU_ERR_NOENT;
     }
   
   status = mailbox_get_message (mbox, msgno, msg);
@@ -1339,9 +1346,12 @@ util_rfc2047_decode (char **value)
     return;
   if (strcasecmp (charset, "auto") == 0)
     {
-      /* Try to deduce the charset from LC_ALL variable */
+      /* Try to deduce the charset from LC_ALL or LANG variables */
 
       tmp = getenv ("LC_ALL");
+      if (!tmp)
+	tmp = getenv ("LANG");
+
       if (tmp)
 	{
 	  char *sp;
