@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2004 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -109,23 +109,15 @@ mailbox_create (mailbox_t *pmbox, const char *name)
 
       /* Create the folder before initializing the concrete mailbox.
 	 The mailbox needs it's back pointer. */
-      {
-	folder_t folder;
-	if (folder_create (&folder, name) == 0)
-	  mbox->folder = folder;
-      }
+      status = folder_create (&mbox->folder, name);
+
+      if (status == 0)
+	status = m_init (mbox);   /* Create the concrete mailbox type.  */
 
-      /* Create the concrete mailbox type.  */
-      status = m_init (mbox);
       if (status != 0)
-	{
-	  mailbox_destroy (&mbox);
-	}
+	mailbox_destroy (&mbox);
       else
-	{
-	  *pmbox = mbox;
-	}
-
+	*pmbox = mbox;
     }
   else
     status = MU_ERR_NO_HANDLER;
@@ -211,6 +203,8 @@ mailbox_flush (mailbox_t mbox, int expunge)
   size_t i, total = 0;
   int status = 0;
 
+  if (!mbox)
+    return EINVAL;
   if (!(mbox->flags & (MU_STREAM_RDWR|MU_STREAM_WRITE|MU_STREAM_APPEND)))
     return EACCES;
   mailbox_messages_count (mbox, &total);
@@ -263,6 +257,7 @@ mailbox_messages_recent (mailbox_t mbox, size_t *num)
     return MU_ERR_EMPTY_VFN;
   return mbox->_messages_recent (mbox, num);
 }
+
 int
 mailbox_message_unseen (mailbox_t mbox, size_t *num)
 {
@@ -498,3 +493,20 @@ mailbox_set_folder (mailbox_t mbox, folder_t folder)
    mbox->folder = folder;
   return 0;
 }
+
+int
+mailbox_lock (mailbox_t mbox)
+{
+  locker_t lock = NULL;
+  mailbox_get_locker (mbox, &lock);
+  return locker_lock (lock);
+}
+
+int
+mailbox_unlock (mailbox_t mbox)
+{
+  locker_t lock = NULL;
+  mailbox_get_locker (mbox, &lock);
+  return locker_unlock (lock);
+}
+  
