@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2004 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -56,7 +56,8 @@ static int message_read   __P ((stream_t is, char *buf, size_t buflen,
 				off_t off, size_t *pnread ));
 static int message_write  __P ((stream_t os, const char *buf, size_t buflen,
 				off_t off, size_t *pnwrite));
-static int message_get_fd __P ((stream_t stream, int *pfd, int *pfd2));
+static int message_get_transport2 __P ((stream_t stream, mu_transport_t *pin, 
+                                       mu_transport_t *pout));
 static int message_sender __P ((envelope_t envelope, char *buf, size_t len,
 				size_t *pnwrite));
 static int message_date   __P ((envelope_t envelope, char *buf, size_t len,
@@ -76,7 +77,7 @@ message_create (message_t *pmsg, void *owner)
   int status;
 
   if (pmsg == NULL)
-    return EINVAL;
+    return MU_ERR_OUT_PTR_NULL;
   msg = calloc (1, sizeof (*msg));
   if (msg == NULL)
     return ENOMEM;
@@ -173,7 +174,9 @@ message_create_copy (message_t *to, message_t from)
   size_t n = 0;
   char buf[512];
 
-  if(!to || !from)
+  if (!to)
+    return MU_ERR_OUT_PTR_NULL;
+  if (!from)
     return EINVAL;
 
   if((status = message_create (to, NULL)))
@@ -249,8 +252,10 @@ message_clear_modified (message_t msg)
 int
 message_get_mailbox (message_t msg, mailbox_t *pmailbox)
 {
-  if (msg == NULL || pmailbox == NULL)
+  if (msg == NULL)
     return EINVAL;
+  if (pmailbox == NULL)
+    return MU_ERR_OUT_PTR_NULL;
   *pmailbox = msg->mailbox;
   return 0;
 }
@@ -269,8 +274,10 @@ message_set_mailbox (message_t msg, mailbox_t mailbox, void *owner)
 int
 message_get_header (message_t msg, header_t *phdr)
 {
-  if (msg == NULL || phdr == NULL)
+  if (msg == NULL)
     return EINVAL;
+  if (phdr == NULL)
+    return MU_ERR_OUT_PTR_NULL;
 
   /* Is it a floating mesg */
   if (msg->header == NULL)
@@ -311,8 +318,10 @@ message_set_header (message_t msg, header_t hdr, void *owner)
 int
 message_get_body (message_t msg, body_t *pbody)
 {
-  if (msg == NULL || pbody == NULL)
+  if (msg == NULL)
     return EINVAL;
+  if (pbody == NULL)
+    return MU_ERR_OUT_PTR_NULL;
 
   /* Is it a floating mesg.  */
   if (msg->body == NULL)
@@ -382,8 +391,10 @@ message_set_stream (message_t msg, stream_t stream, void *owner)
 int
 message_get_stream (message_t msg, stream_t *pstream)
 {
-  if (msg == NULL || pstream == NULL)
+  if (msg == NULL)
     return EINVAL;
+  if (pstream == NULL)
+    return MU_ERR_OUT_PTR_NULL;
 
   if (msg->stream == NULL)
     {
@@ -394,7 +405,7 @@ message_get_stream (message_t msg, stream_t *pstream)
 	return status;
       stream_set_read (stream, message_read, msg);
       stream_set_write (stream, message_write, msg);
-      stream_set_fd (stream, message_get_fd, msg);
+      stream_set_get_transport2 (stream, message_get_transport2, msg);
       stream_set_size (stream, message_stream_size, msg);
       stream_set_flags (stream, MU_STREAM_RDWR);
       msg->stream = stream;
@@ -478,8 +489,10 @@ message_size (message_t msg, size_t *psize)
 int
 message_get_envelope (message_t msg, envelope_t *penvelope)
 {
-  if (msg == NULL || penvelope == NULL)
+  if (msg == NULL)
     return EINVAL;
+  if (penvelope == NULL)
+    return MU_ERR_OUT_PTR_NULL;
 
   if (msg->envelope == NULL)
     {
@@ -512,9 +525,10 @@ message_set_envelope (message_t msg, envelope_t envelope, void *owner)
 int
 message_get_attribute (message_t msg, attribute_t *pattribute)
 {
-  if (msg == NULL || pattribute == NULL)
-   return EINVAL;
-
+  if (msg == NULL)
+    return EINVAL;
+  if (pattribute == NULL)
+    return MU_ERR_OUT_PTR_NULL;
   if (msg->attribute == NULL)
     {
       attribute_t attribute;
@@ -880,9 +894,8 @@ message_write (stream_t os, const char *buf, size_t buflen,
   return status;
 }
 
-/* Implements the stream_get_fd () on the message stream.  */
 static int
-message_get_fd (stream_t stream, int *pfd, int *pfd2)
+message_get_transport2 (stream_t stream, mu_transport_t *pin, mu_transport_t *pout)
 {
   message_t msg = stream_get_owner (stream);
   body_t body;
@@ -890,8 +903,8 @@ message_get_fd (stream_t stream, int *pfd, int *pfd2)
 
   if (msg == NULL)
     return EINVAL;
-  if (pfd2)
-    return ENOSYS;
+  if (pout)
+    *pout = NULL;
 
   /* Probably being lazy, then create a body for the stream.  */
   if (msg->body == NULL)
@@ -905,7 +918,7 @@ message_get_fd (stream_t stream, int *pfd, int *pfd2)
       body = msg->body;
 
   body_get_stream (body, &is);
-  return stream_get_fd (is, pfd);
+  return stream_get_transport2 (is, pin, pout);
 }
 
 /* Implements the stream_stream_size () on the message stream.  */
