@@ -49,6 +49,11 @@
 #include <mailutils/header.h>
 #include <mailutils/message.h>
 #include <mailutils/envelope.h>
+#include <mailutils/nls.h>
+#include <mailutils/stream.h>
+#include <mailutils/filter.h>
+
+#include <registrar0.h>
 
 /* convert a sequence of hex characters into an integer */
 
@@ -392,6 +397,7 @@ mu_get_user_email (const char *name)
   char *localpart = NULL;
   const char *domainpart = NULL;
   char *email = NULL;
+  char *tmpname = NULL;
 
   if (!name && mu_user_email)
     {
@@ -403,25 +409,29 @@ mu_get_user_email (const char *name)
 
   if (!name)
     {
-      struct passwd *pw = getpwuid (getuid ());
-      if (!pw)
+      struct mu_auth_data *auth = mu_get_auth_by_uid (getuid ());
+      if (!auth)
 	{
 	  errno = EINVAL;
 	  return NULL;
 	}
-      name = pw->pw_name;
+      name = tmpname = strdup(auth->name);
+      if (auth)
+	mu_auth_data_free (auth);
     }
 
   status = mu_get_user_email_domain (&domainpart);
 
   if (status)
     {
+      free(tmpname);
       errno = status;
       return NULL;
     }
 
   if ((status = parse822_quote_local_part (&localpart, name)))
     {
+      free(tmpname);
       errno = status;
       return NULL;
     }
@@ -434,6 +444,7 @@ mu_get_user_email (const char *name)
   else
     sprintf (email, "%s@%s", localpart, domainpart);
 
+  free(tmpname);
   free (localpart);
 
   return email;
@@ -565,7 +576,7 @@ mu_tempfile (const char *tmpdir, char **namep)
 
   if (fd == -1)
     {
-      mu_error ("Can not open temporary file: %s", strerror(errno));
+      mu_error (_("Cannot open temporary file: %s"), strerror(errno));
       free (filename);
       return -1;
     }
@@ -758,6 +769,7 @@ mu_unroll_symlink (char *out, size_t outsz, const char *in)
    ---------+------------
    %u         user name
    %h         user's home dir
+   ~          Likewise
    ---------+------------
 
    Allocates memory. 
@@ -773,7 +785,17 @@ mu_expand_path_pattern (const char *pattern, const char *username)
   
   for (p = pattern; *p; p++)
     {
-      if (*p == '%')
+      if (*p == '~')
+        {
+          if (!auth)
+            {
+              auth = mu_get_auth_by_name (username);
+              if (!auth)
+                return NULL;
+            }
+          len += strlen (auth->dir);
+        }
+      else if (*p == '%')
 	switch (*++p)
 	  {
 	  case 'u':
@@ -1114,3 +1136,136 @@ mu_string_unfold (char *text, size_t *plen)
     *plen = p - text;
   return 0;
 }
+
+int
+mu_true_answer_p (const char *p)
+{
+  if (!p)
+    return -1;
+
+  /* TRANSLATORS: This is a list of characters which start
+     an affirmative answer. Whenever possible, please preserve
+     'yY' in your translation, e.g., for Euskara:
+
+     msgstr "yYbB";
+  */
+  if (strchr (_("yY"), *p))
+    return 1;
+
+  /* TRANSLATORS: This is a list of characters which start
+     a negative answer. Whenever possible, please preserve
+     'nN' in your translation, e.g., for Euskara:
+
+     msgstr "nNeE";
+  */
+  else if (strchr (_("nN"), *p))
+    return 0;
+  return -1;
+}
+
+/* Returns true if SCHEME represents a local mail folder. Stores
+   real folder path to PATH */
+int
+mu_scheme_autodetect_p (const char *scheme,  const char **path)
+{
+  *path = scheme;
+  if (strncmp (MU_FILE_SCHEME, scheme, MU_FILE_SCHEME_LEN) == 0)
+    {
+      *path += MU_FILE_SCHEME_LEN;
+      return 1;
+    }
+  if (strncmp (MU_PATH_SCHEME, scheme, MU_PATH_SCHEME_LEN) == 0)
+    return 1;
+  return 0;
+}
+    
+int
+mu_fd_wait (int fd, int *pflags, struct timeval *tvp)
+{
+  fd_set rdset, wrset;
+  int rc;
+
+  FD_ZERO (&rdset);
+  FD_ZERO (&wrset);
+  if ((*pflags) & MU_STREAM_READY_RD)
+    FD_SET (fd, &rdset);
+  if ((*pflags) & MU_STREAM_READY_WR)
+    FD_SET (fd, &wrset);
+  
+  do
+    {
+      if (tvp)
+	{
+	  struct timeval tv = *tvp; 
+	  rc = select (fd + 1, &rdset, &wrset, NULL, &tv);
+	}
+      else
+	rc = select (fd + 1, &rdset, &wrset, NULL, NULL);
+    }
+  while (rc == -1 && errno == EINTR);
+
+  if (rc < 0)
+    return errno;
+  
+  *pflags = 0;
+  if (rc > 0)
+    {
+      if (FD_ISSET (fd, &rdset))
+	*pflags |= MU_STREAM_READY_RD;
+      if (FD_ISSET (fd, &wrset))
+	*pflags |= MU_STREAM_READY_WR;
+    }
+  return 0;
+}
+
+enum mu_iconv_fallback_mode mu_default_fallback_mode = mu_fallback_copy_octal;
+
+int
+mu_set_default_fallback (const char *str)
+{
+  if (strcmp (str, "none") == 0)
+    mu_default_fallback_mode = mu_fallback_none;
+  else if (strcmp (str, "copy-pass") == 0)
+    mu_default_fallback_mode = mu_fallback_copy_pass;
+  else if (strcmp (str, "copy-octal") == 0)
+    mu_default_fallback_mode = mu_fallback_copy_octal;
+  else
+    return EINVAL;
+  return 0;
+}
+
+int
+mu_decode_filter (stream_t *pfilter, stream_t input,
+		  const char *filter_type,
+		  const char *fromcode, const char *tocode)
+{
+  stream_t filter;
+  
+  int status = filter_create (&filter, input, filter_type,
+			      MU_FILTER_DECODE, MU_STREAM_READ);
+  if (status)
+    return status;
+
+  if (fromcode && tocode && strcasecmp (fromcode, tocode))
+    {
+      stream_t cvt;
+      status = filter_iconv_create (&cvt, filter, fromcode, tocode,
+				    MU_STREAM_NO_CLOSE,
+				    mu_default_fallback_mode);
+      if (status == 0)
+	{
+	  if (stream_open (cvt))
+	    stream_destroy (&cvt, stream_get_owner (cvt));
+	  else
+	    {
+	      int flags;
+	      stream_get_flags (cvt, &flags);
+	      flags &= ~MU_STREAM_NO_CLOSE;
+	      stream_set_flags (cvt, flags);
+	      filter = cvt;
+	    }
+	}
+    }
+  *pfilter = filter;
+  return 0;
+}
