@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2004 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -500,8 +500,8 @@ message_has_bcc (message_t msg)
 
   status = header_get_value (header, MU_HEADER_BCC, NULL, 0, &bccsz);
 
-  /* ENOENT, or there was a Bcc: field. */
-  return status == ENOENT ? 0 : 1;
+  /* MU_ERR_NOENT, or there was a Bcc: field. */
+  return status == MU_ERR_NOENT ? 0 : 1;
 }
 
 /*
@@ -596,7 +596,7 @@ smtp_send_message (mailer_t mailer, message_t argmsg, address_t argfrom,
     case SMTP_ENV_FROM:
     ENV_FROM:
       {
-	status = smtp_writeline (smtp, "MAIL FROM: %s\r\n", smtp->mail_from);
+	status = smtp_writeline (smtp, "MAIL FROM: <%s>\r\n", smtp->mail_from);
 	CHECK_ERROR (smtp, status);
 	smtp->state = SMTP_MAIL_FROM;
       }
@@ -639,7 +639,7 @@ smtp_send_message (mailer_t mailer, message_t argmsg, address_t argfrom,
 	    CHECK_ERROR (smtp, status);
 	  }
 
-	status = smtp_writeline (smtp, "RCPT TO: %s\r\n", to);
+	status = smtp_writeline (smtp, "RCPT TO: <%s>\r\n", to);
 
 	free (to);
 
@@ -708,21 +708,30 @@ smtp_send_message (mailer_t mailer, message_t argmsg, address_t argfrom,
 
 	message_get_header (smtp->msg, &hdr);
 	header_get_stream (hdr, &stream);
-	while ((status = stream_readline (stream, data, sizeof (data) - 1,
+	while ((status = stream_readline (stream, data, sizeof (data),
 					  smtp->offset, &n)) == 0 && n > 0)
 	  {
+	    int nl;
+	    
 	    found_nl = (n == 1 && data[0] == '\n');
-	    if (data[n - 1] == '\n')
+	    if ((nl = (data[n - 1] == '\n')))
 	      data[n - 1] = '\0';
 	    if (data[0] == '.')
 	      {
-		status = smtp_writeline (smtp, ".%s\r\n", data);
+		status = smtp_writeline (smtp, ".%s", data);
 		CHECK_ERROR (smtp, status);
 	      }
 	    else if (strncasecmp (data, MU_HEADER_FCC,
 				  sizeof (MU_HEADER_FCC) - 1))
 	      {
-		status = smtp_writeline (smtp, "%s\r\n", data);
+		status = smtp_writeline (smtp, "%s", data);
+		CHECK_ERROR (smtp, status);
+		status = smtp_write (smtp);
+		CHECK_EAGAIN (smtp, status);
+	      }
+	    if (nl)
+	      {
+		status = smtp_writeline (smtp, "\r\n");
 		CHECK_ERROR (smtp, status);
 		status = smtp_write (smtp);
 		CHECK_EAGAIN (smtp, status);
@@ -865,7 +874,7 @@ _smtp_set_from (smtp_t smtp, message_t msg, address_t from)
 	  }
 	  break;
 
-	case ENOENT:
+	case MU_ERR_NOENT:
 	  /* Use the environment. */
 	  mail_from = mu_get_user_email (NULL);
 
@@ -914,13 +923,22 @@ smtp_address_add (address_t *paddr, const char *value)
   return status;
 }
 
+static int
+_smtp_property_is_set (smtp_t smtp, const char *name)
+{
+  property_t property = NULL;
+
+  mailer_get_property (smtp->mailer, &property);
+  return property_is_set (property, name);
+}
+
 static int
 _smtp_set_rcpt (smtp_t smtp, message_t msg, address_t to)
 {
   int status = 0;
   header_t header = NULL;
   char *value;
-  
+
   /* Get RCPT_TO from TO, or the message. */
 
   if (to)
@@ -939,45 +957,48 @@ _smtp_set_rcpt (smtp_t smtp, message_t msg, address_t to)
 	return status;
     }
 
-  if ((status = message_get_header (msg, &header)))
-    return status;
-
-  status = header_aget_value (header, MU_HEADER_TO, &value);
-
-  if (status == 0)
+  if (!to || _smtp_property_is_set (smtp, "READ_RECIPIENTS"))
     {
-      smtp_address_add (&smtp->rcpt_to, value);
-      free (value);
+      if ((status = message_get_header (msg, &header)))
+	return status;
+
+      status = header_aget_value (header, MU_HEADER_TO, &value);
+
+      if (status == 0)
+	{
+	  smtp_address_add (&smtp->rcpt_to, value);
+	  free (value);
+	}
+      else if (status != MU_ERR_NOENT)
+	goto end;
+
+      status = header_aget_value (header, MU_HEADER_CC, &value);
+
+      if (status == 0)
+	{
+	  smtp_address_add (&smtp->rcpt_to, value);
+	  free (value);
+	}
+      else if (status != MU_ERR_NOENT)
+	goto end;
+
+      status = header_aget_value (header, MU_HEADER_BCC, &value);
+      if (status == 0)
+	{
+	  smtp_address_add (&smtp->rcpt_bcc, value);
+	  free (value);
+	}
+      else if (status != MU_ERR_NOENT)
+	goto end;
+
+      /* If to or bcc is present, the must be OK. */
+      if (smtp->rcpt_to && (status = mailer_check_to (smtp->rcpt_to)))
+	goto end;
+
+      if (smtp->rcpt_bcc && (status = mailer_check_to (smtp->rcpt_bcc)))
+	goto end;
     }
-  else if (status && status != ENOENT)
-    goto end;
-
-  status = header_aget_value (header, MU_HEADER_CC, &value);
-
-  if (status == 0)
-    {
-      smtp_address_add (&smtp->rcpt_to, value);
-      free (value);
-    }
-  else if (status && status != ENOENT)
-    goto end;
-
-  status = header_aget_value (header, MU_HEADER_BCC, &value);
-  if (status == 0)
-    {
-      smtp_address_add (&smtp->rcpt_bcc, value);
-      free (value);
-    }
-  else if (status && status != ENOENT)
-    goto end;
-
-  /* If to or bcc is present, the must be OK. */
-  if (smtp->rcpt_to && (status = mailer_check_to (smtp->rcpt_to)))
-    goto end;
-
-  if (smtp->rcpt_bcc && (status = mailer_check_to (smtp->rcpt_bcc)))
-    goto end;
-
+  
 end:
 
   if (status)
