@@ -25,9 +25,11 @@
 #include <fcntl.h>
 
 const char *program_version = "refile (" PACKAGE_STRING ")";
-static char doc[] = N_("GNU MH refile\v"
-"Options marked with `*' are not yet implemented.\n"
-"Use -help to obtain the list of traditional MH options.");
+/* TRANSLATORS: Please, preserve the vertical tabulation (^K character)
+   in this message */
+static char doc[] = N_("GNU MH refile\v\
+Options marked with `*' are not yet implemented.\n\
+Use -help to obtain the list of traditional MH options.");
 static char args_doc[] = N_("messages folder [folder...]");
 
 /* GNU options */
@@ -230,7 +232,7 @@ main (int argc, char **argv)
   mu_init_nls ();
 
   mu_argp_init (program_version, NULL);
-  mh_argp_parse (argc, argv, 0, options, mh_option, args_doc, doc,
+  mh_argp_parse (&argc, &argv, 0, options, mh_option, args_doc, doc,
 		 opt_handler, NULL, &index);
 
   argc -= index;
