@@ -22,9 +22,11 @@
 #include <unistd.h>
 
 const char *program_version = "reply (" PACKAGE_STRING ")";
-static char doc[] = N_("GNU MH repl\v"
-"Options marked with `*' are not yet implemented.\n"
-"Use -help to obtain the list of traditional MH options.");
+/* TRANSLATORS: Please, preserve the vertical tabulation (^K character)
+   in this message */
+static char doc[] = N_("GNU MH repl\v\
+Options marked with `*' are not yet implemented.\n\
+Use -help to obtain the list of traditional MH options.");
 static char args_doc[] = N_("[+folder] [msg]");
 
 
@@ -45,16 +47,20 @@ static struct argp_option options[] = {
   {"nocc", ARG_NOCC, "{all|to|cc|me}", 0,
    N_("Specify whom to remove from the Cc: list of the reply")},
   {"folder",  ARG_FOLDER, N_("FOLDER"), 0, N_("Specify folder to operate upon")},
+  {"group",  ARG_GROUP,  N_("BOOL"), OPTION_ARG_OPTIONAL,
+   N_("Construct a group or followup reply") },
   {"editor", ARG_EDITOR, N_("PROG"), 0, N_("Set the editor program to use")},
   {"noedit", ARG_NOEDIT, 0, 0, N_("Suppress the initial edit")},
   {"fcc",    ARG_FCC, N_("FOLDER"), 0, N_("* Set the folder to receive Fcc's.")},
   {"filter", ARG_FILTER, N_("MHL-FILTER"), 0,
    N_("Set the mhl filter to preprocess the body of the message being replied")},
   {"form",   ARG_FORM, N_("FILE"), 0, N_("Read format from given file")},
+  {"format", ARG_FORMAT, N_("BOOL"), OPTION_ARG_OPTIONAL,
+   N_("Include a copy of the message being replied. The message will be processed using either the default filter \"mhl.reply\", or the filter specified by --filter option.") },
   {"inplace", ARG_INPLACE, N_("BOOL"), OPTION_ARG_OPTIONAL,
    N_("* Annotate the message in place")},
   {"query", ARG_QUERY, N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("* Query for addresses to place in To: and Cc: lists")},
+   N_("Query for addresses to place in To: and Cc: lists")},
   {"width", ARG_WIDTH, N_("NUMBER"), 0, N_("Set output width")},
   {"whatnowproc", ARG_WHATNOWPROC, N_("PROG"), 0,
    N_("* Set the replacement for whatnow program")},
@@ -81,6 +87,8 @@ struct mh_option mh_option[] = {
   {"noedit",      3, },
   {"fcc",         1, MH_OPT_ARG, "folder"},
   {"filter",      2, MH_OPT_ARG, "program"},
+  {"format",      2, MH_OPT_BOOL },
+  {"group",       1, MH_OPT_BOOL },
   {"inplace",     1, MH_OPT_BOOL },
   {"query",       1, MH_OPT_BOOL },
   {"whatnowproc", 2, MH_OPT_ARG, "program"},
@@ -88,7 +96,7 @@ struct mh_option mh_option[] = {
   { 0 }
 };
 
-static char *format_str =
+static char default_format_str[] =
 "%(lit)%(formataddr %<{reply-to}%?{from}%?{sender}%?{return-path}%>)"
 "%<(nonnull)%(void(width))%(putaddr To: )\\n%>"
 "%(lit)%<(rcpt to)%(formataddr{to})%>%<(rcpt cc)%(formataddr{cc})%>%<(rcpt me)%(formataddr(me))%>"
@@ -100,6 +108,7 @@ static char *format_str =
 "X-Mailer: MH \\(%(package_string)\\)\\n" 
 "--------\n";
 
+static char *format_str = NULL;
 static mh_format_t format;
 static int width = 80;
 
@@ -156,11 +165,28 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       break;
 
     case ARG_FORM:
+      free (format_str);
+      format_str = NULL;
       s = mh_expand_name (MHLIBDIR, arg, 0);
       mh_read_formfile (s, &format_str);
       free (s);
       break;
 
+    case ARG_GROUP:
+      if (is_true (arg))
+	{
+	  s = mh_expand_name (MHLIBDIR, "replgroupcomps", 0);
+	  mh_read_formfile (s, &format_str);
+	  free (s);
+	  rcpt_mask |= RCPT_ALL;
+	}
+      else
+	{
+	  free (format_str);
+	  format_str = NULL;
+	}
+      break;
+	    
     case ARG_DRAFTMESSAGE:
       wh_env.draftmessage = arg;
       break;
@@ -193,6 +219,16 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
     case ARG_FILTER:
       mhl_filter = arg;
       break;
+
+    case ARG_FORMAT:
+      if (is_true (arg))
+	{
+	  if (!mhl_filter)
+	    mhl_filter = mh_expand_name (MHLIBDIR, "mhl.repl", 0);
+	}
+      else
+	mhl_filter = NULL;
+      break;
       
     case ARG_ANNOTATE:
     case ARG_FCC:
@@ -206,6 +242,11 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       mh_license (argp_program_version);
       break;
 
+    case ARGP_KEY_FINI:
+      if (!format_str)
+	format_str = default_format_str;
+      break;
+      
     default:
       return 1;
     }
@@ -324,7 +365,7 @@ main (int argc, char **argv)
   mu_init_nls ();
 
   mu_argp_init (program_version, NULL);
-  mh_argp_parse (argc, argv, 0, options, mh_option, args_doc, doc,
+  mh_argp_parse (&argc, &argv, 0, options, mh_option, args_doc, doc,
 		 opt_handler, NULL, &index);
   if (mh_format_parse (format_str, &format))
     {
