@@ -38,7 +38,7 @@ pop3d_dele (const char *arg)
     return ERR_NO_MESG;
 
   message_get_attribute (msg, &attr);
-  attribute_set_deleted (attr);
+  pop3d_mark_deleted (attr);
   pop3d_outf ("+OK Message %d marked\r\n", num);
   return OK;
 }
