@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@ struct daemon_param daemon_param = {
   110,                  /* Standard POP3 port */
   600,                  /* Idle timeout */
   0,                    /* No transcript by default */
+  NULL                  /* No PID file by default */
 };
 
 int debug_mode;
@@ -39,6 +40,16 @@ int tls_done;
 
 /* Number of child processes.  */
 volatile size_t children;
+/* Should all the messages be undeleted on startup */
+int undelete_on_startup;
+#ifdef ENABLE_LOGIN_DELAY
+/* Minimum allowed delay between two successive logins */
+time_t login_delay = 0;
+char *login_stat_file = LOGIN_STAT_FILE;
+#endif
+
+/* Minimum advertise retention times of messages.  */
+int expire = -1;
 
 static int pop3d_mainloop       __P ((int fd, FILE *, FILE *));
 static void pop3d_daemon_init   __P ((void));
@@ -50,8 +61,26 @@ static void pop3d_log_connection __P((int fd));
 const char *program_version = "pop3d (" PACKAGE_STRING ")";
 static char doc[] = N_("GNU pop3d -- the POP3 daemon");
 
+#define OPT_LOGIN_DELAY 257
+#define OPT_STAT_FILE   258
+#define OPT_EXPIRE      259
+
+static struct argp_option options[] = {
+  {"undelete", 'u', NULL, 0,
+   N_("Undelete all messages on startup"), 0},
+#ifdef ENABLE_LOGIN_DELAY
+  {"login-delay", OPT_LOGIN_DELAY, N_("SECONDS"), 0,
+   N_("Allowed delay between the two successive logins"), 0},
+  {"stat-file", OPT_STAT_FILE, N_("FILENAME"), 0,
+   N_("Name of login statistics file"), 0},
+#endif
+  {"expire", OPT_EXPIRE, N_("DAYS"), 0,
+   N_("Maximum retention period for messages in the maildrop, default -1 means NEVER"), 0},
+  {NULL, 0, NULL, 0, NULL, 0}
+};
+
 static struct argp argp = {
-  NULL,
+  options,
   pop3d_parse_opt,
   NULL,
   doc,
@@ -75,11 +104,41 @@ static const char *pop3d_argp_capa[] = {
 static error_t
 pop3d_parse_opt (int key, char *arg, struct argp_state *astate)
 {
+  char *p;
+  
   switch (key)
     {
     case ARGP_KEY_INIT:
       astate->child_inputs[0] = astate->input;
       break;
+
+    case 'u':
+      undelete_on_startup = 1;
+      break;
+
+#ifdef ENABLE_LOGIN_DELAY
+    case OPT_LOGIN_DELAY:
+      login_delay = strtoul (arg, &p, 10);
+      if (*p)
+	{
+	  argp_error (astate, _("Invalid number"));
+	  exit (1);
+	}
+      break;
+
+    case OPT_STAT_FILE:
+      login_stat_file = arg;
+      break;
+#endif  
+ 
+    case OPT_EXPIRE:
+      expire = strtoul (arg, &p, 10);
+      if (*p)
+	{
+	  argp_error (astate, _("Invalid number"));
+	  exit (1);
+	}
+      break;
       
     default:
       return ARGP_ERR_UNKNOWN;
@@ -131,12 +190,7 @@ main (int argc, char **argv)
     }
 
   /* Register the desired formats.  */
-  {
-    list_t bookie;
-    registrar_get_list (&bookie);
-    list_append (bookie, mbox_record);
-    list_append (bookie, path_record);
-  }
+  mu_register_local_mbox_formats ();
 
   /* Set the signal handlers.  */
   signal (SIGINT, pop3d_signal);
@@ -166,6 +220,11 @@ main (int argc, char **argv)
   
   umask (S_IROTH | S_IWOTH | S_IXOTH);	/* 007 */
 
+  if (daemon_param.pidfile)
+    {
+      daemon_create_pidfile (daemon_param.pidfile);
+    }
+
   /* Check TLS environment, i.e. cert and key files */
 #ifdef WITH_TLS
   tls_available = mu_check_tls_environment ();
@@ -351,7 +410,14 @@ pop3d_mainloop (int fd, FILE *infile, FILE *outfile)
 	status = pop3d_capa (arg);
 #ifdef WITH_TLS
       else if ((strncasecmp (cmd, "STLS", 4) == 0) && tls_available)
-	status = pop3d_stls (arg);
+	{
+	  status = pop3d_stls (arg);
+	  if (status)
+	    {
+	      syslog (LOG_ERR, _("Session terminated"));
+	      break;
+	    }
+	}
 #endif /* WITH_TLS */
       else
 	status = ERR_BAD_CMD;
@@ -382,6 +448,8 @@ pop3d_mainloop (int fd, FILE *infile, FILE *outfile)
       else if (status == ERR_TLS_ACTIVE)
 	pop3d_outf ("-ERR " TLS_ACTIVE "\r\n");
 #endif /* WITH_TLS */
+      else if (status == ERR_LOGIN_DELAY)
+	pop3d_outf ("-ERR [LOGIN-DELAY] " LOGIN_DELAY "\r\n");
       else
 	pop3d_outf ("-ERR unknown error\r\n");
 
@@ -406,7 +474,7 @@ pop3d_daemon (unsigned int maxchildren, unsigned int port)
   int listenfd, connfd;
   size_t size;
 
-  listenfd = socket (AF_INET, SOCK_STREAM, 0);
+  listenfd = socket (PF_INET, SOCK_STREAM, 0);
   if (listenfd == -1)
     {
       syslog (LOG_ERR, "socket: %s", strerror(errno));
@@ -432,8 +500,11 @@ pop3d_daemon (unsigned int maxchildren, unsigned int port)
       exit (EXIT_FAILURE);
     }
 
+  syslog (LOG_INFO, _("GNU pop3d started"));
+
   for (;;)
     {
+      process_cleanup ();
       if (children > maxchildren)
         {
 	  syslog (LOG_ERR, _("too many children (%lu)"),
