@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -62,16 +62,37 @@
 /* Command not permitted when TLS active. */
 #define TLS_ACTIVE      "Command not permitted when TLS active"
 
+/* Trying to log in within the minimum login delay interval */
+#define LOGIN_DELAY     "Attempt to log in within the minimum login delay interval"
+
 /* APOP password file, without .db or .passwd, which are added based on file
    type automatically */
 #define APOP_PASSFILE_NAME "apop"
 
 #ifdef USE_DBM
 # define APOP_PASSFILE SYSCONFDIR "/" APOP_PASSFILE_NAME
+# define ENABLE_LOGIN_DELAY
 #else
 # define APOP_PASSFILE SYSCONFDIR "/" APOP_PASSFILE_NAME ".passwd"
+# undef ENABLE_LOGIN_DELAY
 #endif
 
+#ifdef ENABLE_LOGIN_DELAY
+# define LOGIN_STAT_FILE "/var/run/pop3-login"
+extern time_t login_delay;
+extern char *login_stat_file;
+extern int check_login_delay __P((char *username));
+extern void update_login_delay __P((char *username));
+extern void login_delay_capa __P((void));
+#else
+# define check_login_delay(u) 0
+# define update_login_delay(u)
+# define login_delay_capa()
+#endif
+
+/* Minimum advertise retention time for messages.  */
+extern int expire;
+
 /* Size of the MD5 digest for APOP */
 #define APOP_DIGEST	70
 
@@ -115,6 +136,7 @@
 #include <mailutils/argp.h>
 #include <mailutils/attribute.h>
 #include <mailutils/body.h>
+#include <mailutils/daemon.h>
 #include <mailutils/errno.h>
 #include <mailutils/error.h>
 #include <mailutils/header.h>
@@ -123,11 +145,11 @@
 #include <mailutils/mailbox.h>
 #include <mailutils/message.h>
 #include <mailutils/mutil.h>
-#include <mailutils/registrar.h>
-#include <mailutils/url.h>
 #include <mailutils/mu_auth.h>
-#include <mailutils/tls.h>
 #include <mailutils/nls.h>
+#include <mailutils/registrar.h>
+#include <mailutils/tls.h>
+#include <mailutils/url.h>
 
 /* For Berkley DB2 APOP password file */
 #ifdef HAVE_DB_H
@@ -144,10 +166,12 @@
 
 #ifndef MAXHOSTNAMELEN
 /* Maximum length of a hostname (is this defined somewhere else?).  */
-/* MAXHOSTNAMELEN is already define on Solaris.  */
-#define MAXHOSTNAMELEN	64
+/* MAXHOSTNAMELEN is already defined on Solaris.  */
+# define MAXHOSTNAMELEN	64
 #endif
 
+#define POP3_ATTRIBUTE_DELE 0x0001
+#define POP3_ATTRIBUTE_RETR 0x0010
 
 #define AUTHORIZATION	0
 #define TRANSACTION	1
@@ -172,6 +196,7 @@
 #define ERR_MBOX_SYNC   16
 #define ERR_TLS_ACTIVE  17
 #define ERR_TLS_IO      18
+#define ERR_LOGIN_DELAY 19
 
 extern mailbox_t mbox;
 extern int state;
@@ -185,6 +210,7 @@ extern int debug_mode;
 extern int tls_available;
 extern int tls_done;
 #endif /* WITH_TLS */
+extern int undelete_on_startup;
 
 extern void pop3d_bye           __P ((void));
 extern int pop3d_abquit         __P ((int));
@@ -222,9 +248,18 @@ extern void pop3d_flush_output  __P ((void));
 
 extern int pop3d_is_master      __P ((void));
 
+extern void pop3d_mark_deleted __P((attribute_t attr));
+extern int pop3d_is_deleted __P((attribute_t attr));
+extern void pop3d_unset_deleted __P((attribute_t attr));
+void pop3d_undelete_all __P((void));
+
 #ifdef WITH_TLS
 extern int pop3d_init_tls_server    __P ((void));
 extern void pop3d_deinit_tls_server __P ((void));
 #endif /* WITH_TLS */
 
+extern void pop3d_mark_retr __P((attribute_t attr));
+extern int pop3d_is_retr __P((attribute_t attr));
+extern void pop3d_unmark_retr __P((attribute_t attr));
+
 #endif /* _POP3D_H */
