@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -42,27 +42,30 @@
 #include <mailutils/registrar.h>
 #include <mailutils/stream.h>
 #include <mailutils/nls.h>
-
-void mutil_register_all_mbox_formats (void);
+#include <mailutils/tls.h>
 
 const char *program_version = "sieve (" PACKAGE_STRING ")";
 
 static char doc[] =
-N_("GNU sieve -- a mail filtering tool\n"
-"\v"
-"Debug flags:\n"
-"  g - main parser traces\n"
-"  T - mailutil traces (MU_DEBUG_TRACE)\n"
-"  P - network protocols (MU_DEBUG_PROT)\n"
-"  t - sieve trace (MU_SIEVE_DEBUG_TRACE)\n"
-"  i - sieve instructions trace (MU_SIEVE_DEBUG_INSTR)\n");
+/* TRANSLATORS: Please, preserve the vertical tabulation (^K character)
+   in this message */
+N_("GNU sieve -- a mail filtering tool\n\
+\v\
+Debug flags:\n\
+  g - main parser traces\n\
+  T - mailutil traces (MU_DEBUG_TRACE)\n\
+  P - network protocols (MU_DEBUG_PROT)\n\
+  t - sieve trace (MU_SIEVE_DEBUG_TRACE)\n\
+  i - sieve instructions trace (MU_SIEVE_DEBUG_INSTR)\n");
 
 #define D_DEFAULT "TPt"
 
+#define ARG_LINE_INFO 257
+
 static struct argp_option options[] =
 {
   {"no-actions", 'n', 0, 0,
-   N_("No actions executed, just print what would be done"), 0},
+   N_("Do not execute any actions, just print what would be done"), 0},
 
   {"keep-going", 'k', 0, 0,
    N_("Keep on going if execution fails on a message"), 0},
@@ -84,7 +87,10 @@ static struct argp_option options[] =
 
   {"verbose", 'v', NULL, 0,
    N_("Log all actions"), 0},
-  
+
+  {"line-info", ARG_LINE_INFO, N_("BOOL"), OPTION_ARG_OPTIONAL,
+   N_("Print source location along with action logs (default)") },
+   
   {"email", 'e', N_("ADDRESS"), 0,
    N_("Override user email address"), 0},
   
@@ -96,12 +102,27 @@ struct options {
   int compile_only;
   char *mbox;
   char *tickets;
+  int tickets_default;
   int debug_level;
   int sieve_debug;
   int verbose;
   char *script;
 };
 
+static int sieve_print_locus = 1; /* Should the log messages include the
+				     locus */
+
+static int
+is_true_p (char *p)
+{
+  int rc;
+  if (!p)
+    return 1;
+  if ((rc = mu_true_answer_p (p)) == -1)
+    return 0;
+  return rc;
+}
+
 static error_t
 parser (int key, char *arg, struct argp_state *state)
 {
@@ -112,7 +133,10 @@ parser (int key, char *arg, struct argp_state *state)
     {
     case ARGP_KEY_INIT:
       if (!opts->tickets)
-	opts->tickets = mu_tilde_expansion ("~/.tickets", "/", NULL);
+	{
+	  opts->tickets = mu_tilde_expansion ("~/.tickets", "/", NULL);
+	  opts->tickets_default = 1;
+	}
       if (!opts->debug_level)
 	opts->debug_level = MU_DEBUG_ERROR;
       log_facility = 0;
@@ -149,6 +173,7 @@ parser (int key, char *arg, struct argp_state *state)
     case 't':
       free (opts->tickets);
       opts->tickets = mu_tilde_expansion (arg, "/", NULL);
+      opts->tickets_default = 0;
       break;
       
     case 'd':
@@ -188,6 +213,10 @@ parser (int key, char *arg, struct argp_state *state)
     case 'v':
       opts->verbose = 1;
       break;
+
+    case ARG_LINE_INFO:
+      sieve_print_locus = is_true_p (arg);
+      break;
       
     case ARGP_KEY_ARG:
       if (opts->script)
@@ -221,6 +250,9 @@ static const char *sieve_argp_capa[] =
   "logging",
   "mailer",
   "sieve",
+#ifdef WITH_TLS
+  "tls",
+#endif
   NULL
 };
 
@@ -256,14 +288,20 @@ syslog_debug_print (mu_debug_t unused, size_t level, const char *fmt,
 
 static void
 stdout_action_log (void *unused,
-		   const char *script, size_t msgno, message_t msg,
+		   const sieve_locus_t *locus, size_t msgno, message_t msg,
 		   const char *action, const char *fmt, va_list ap)
 {
   size_t uid = 0;
-
+  
   message_get_uid (msg, &uid);
 
-  fprintf (stdout, _("%s on msg uid %lu"), action, (unsigned long) uid);
+  if (sieve_print_locus)
+    fprintf (stdout, _("%s:%lu: %s on msg uid %lu"),
+	     locus->source_file, (unsigned long) locus->source_line,
+	     action, (unsigned long) uid);
+  else
+    fprintf (stdout, _("%s on msg uid %lu"), action, (unsigned long) uid);
+  
   if (fmt && strlen (fmt))
     {
       fprintf (stdout, ": ");
@@ -274,7 +312,7 @@ stdout_action_log (void *unused,
 
 static void
 syslog_action_log (void *unused,
-		   const char *script, size_t msgno, message_t msg,
+		   const sieve_locus_t *locus, size_t msgno, message_t msg,
 		   const char *action, const char *fmt, va_list ap)
 {
   size_t uid = 0;
@@ -282,7 +320,13 @@ syslog_action_log (void *unused,
   
   message_get_uid (msg, &uid);
 
-  asprintf (&text, _("%s on msg uid %d"), action, uid);
+  if (sieve_print_locus)
+    asprintf (&text, _("%s:%lu: %s on msg uid %lu"),
+	      locus->source_file, (unsigned long) locus->source_line,
+	      action, (unsigned long) uid);
+  else
+    asprintf (&text, _("%s on msg uid %lu"), action, (unsigned long) uid);
+    
   if (fmt && strlen (fmt))
     {
       char *diag = NULL;
@@ -311,6 +355,9 @@ main (int argc, char *argv[])
   mu_init_nls ();
 
   mu_argp_init (program_version, NULL);
+#ifdef WITH_TLS
+  mu_tls_init_client_argp ();
+#endif
   sieve_argp_init ();
   rc = mu_argp_parse (&argp, &argc, &argv, ARGP_IN_ORDER, sieve_argp_capa,
 		      0, &opts);
@@ -318,7 +365,7 @@ main (int argc, char *argv[])
   if (rc)
     return 1;
 
-  mutil_register_all_mbox_formats ();
+  mu_register_all_formats ();
 
   /* Sieve interpreter setup. */
   rc = sieve_machine_init (&mach, NULL);
@@ -360,18 +407,22 @@ main (int argc, char *argv[])
   /* Create a ticket, if we can. */
   if (opts.tickets)
     {
-      if ((rc = wicket_create (&wicket, opts.tickets)) != 0)
-	{
-	  mu_error (_("wicket_create <%s> failed: %s"),
-		   opts.tickets, mu_strerror (rc));
-	  goto cleanup;
-	}
-      if ((rc = wicket_get_ticket (wicket, &ticket, 0, 0)) != 0)
-	{
-	  mu_error (_("ticket_get failed: %s"), mu_strerror (rc));
-	  goto cleanup;
-	}
-      sieve_set_ticket (mach, ticket);
+      if ((rc = wicket_create (&wicket, opts.tickets)) == 0)
+        {
+          if ((rc = wicket_get_ticket (wicket, &ticket, 0, 0)) != 0)
+            {
+              mu_error (_("ticket_get failed: %s"), mu_strerror (rc));
+              goto cleanup;
+            }
+        }
+      else if (!(opts.tickets_default && errno == ENOENT))
+        {
+          mu_error (_("wicket_create <%s> failed: %s"),
+                    opts.tickets, mu_strerror (rc));
+          goto cleanup;
+        }
+      if (ticket)
+        sieve_set_ticket (mach, ticket);
     }
 
   /* Create a debug object, if needed. */
@@ -450,6 +501,7 @@ main (int argc, char *argv[])
     {
       mu_error (_("open on %s failed: %s"),
 		opts.mbox ? opts.mbox : _("default"), mu_strerror (rc));
+      mailbox_destroy (&mbox);
       goto cleanup;
     }
 
@@ -481,17 +533,3 @@ cleanup:
   return rc ? 1 : 0;
 }
 
-void
-mutil_register_all_mbox_formats (void)
-{
-  list_t bookie = 0;
-  registrar_get_list (&bookie);
-  list_append (bookie, path_record);
-  list_append (bookie, file_record);
-  list_append (bookie, mbox_record);
-  list_append (bookie, pop_record);
-  list_append (bookie, imap_record);
-  list_append (bookie, mh_record);
-  list_append (bookie, sendmail_record);
-  list_append (bookie, smtp_record);
-}
